/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "textured.h"

void shaderInit(shader_t *shader) {
  shader->programCount = 0;
  shader->varCount = 0;

  shaderSetInt32(shader, SHADER_KEY_WIDTH, 32);
  shaderSetInt32(shader, SHADER_KEY_HEIGHT, 32);

  renderListInit(
    &shader->renderList,
    shaderGetInt32(shader, SHADER_KEY_WIDTH),
    shaderGetInt32(shader, SHADER_KEY_HEIGHT)
  );
}

// Setters
int32_t shaderSet(shader_t *shader, char *key, uint8_t type) {
  int32_t i = dictionaryGetOrAdd(
    shader->varNames, key, &shader->varCount,
    SHADER_VARIABLE_NAME, shader->varNameBuffer
  );
  shader->varTypes[i] = type;
  return i;
}

int32_t shaderSetColor(shader_t *shader, char *key, pixel_t color) {
  int32_t i = shaderSet(shader, key, 0x00);
  shader->varData[i].color = color;
  return i;
}

void shaderSetInt32(shader_t *shader, char *key, int32_t v) {
  int32_t i = shaderSet(shader, key, SHADER_VAR_TYPE_I32);
  shader->varData[i].i32 = v;
  return i;
}

// Getters
int32_t shaderGet(shader_t *shader, char *key) {
  return dictionaryGetIndex(shader->varNames, key, shader->varCount);
}

int32_t shaderGetInt32(shader_t *shader, char *key) {
  int32_t i = shaderGet(shader, key);
  if(i == -1) return 0;
  return shader->varData[i].i32;
}