/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../libs.h"
#include "../display/renderlist.h"
#include "../util/dictionary.h"

#define SHADER_VARIABLE_COUNT 128
#define SHADER_VARIABLE_NAME 32
#define SHADER_PROGRAM_COUNT 8

#define SHADER_KEY_WIDTH "width"
#define SHADER_KEY_HEIGHT "height"

#define SHADER_VAR_TYPE_I32 0x00

/** Representation of a shader uniform */
typedef GLuint shaderuniform_t;

typedef union {
  pixel_t color;
  matrix_t matrix;
  int32_t i32;
} shadervariabledata_t;

typedef struct {
  /** Pointer to an uploaded vertex shader program */
  GLuint shaderVertex;

  /** Pointer to an uploaded fragment shader program */
  GLuint shaderFrag;

  /** Pointer to an uploaded shader program linked */
  GLuint shaderProgram;

  /** Buffer of chars where we store the uniform names */
  char uniformBuffer[SHADER_UNIFORM_NAME_MAX * SHADER_UNIFORM_COUNT];

  /** Array of strings (pointers to the above buffer) of the uniform names */
  char *uniformNames[SHADER_UNIFORM_COUNT];
  int32_t uniformCount;

  /** Type of each uniform */
  GLenum types[SHADER_UNIFORM_COUNT];

  /** Texture Slots (which texture slot for GL to use for each uniform) */
  uint8_t textureSlots[SHADER_UNIFORM_COUNT];
} shaderprogram_t;

typedef struct {
  renderlist_t renderList;
  scene_t *scene;
  void *user;

  shaderprogram_t programs[SHADER_PROGRAM_COUNT];
  int32_t programCount;
  
  char varNameBuffer[SHADER_VARIABLE_COUNT * SHADER_VARIABLE_NAME]
  shadervariabledata_t varData[SHADER_VARIABLE_COUNT];
  uint8_t varTypes[SHADER_VARIABLE_COUNT];
  char *varNames[SHADER_VARIABLE_COUNT];
  int32_t varCount;
} shader_t;